from functools import wraps

from allianceauth.services.hooks import get_extension_logger
from esi.errors import TokenError

def fetch_token_for_owner(scopes):
    """returns valid token for owner.
    Needs to be attached on an Owner method !!

    Args:
    -scopes: Provide the required scopes.
    """

    def decorator(func):
        @wraps(func)
        def _wrapped_view(owner, *args, **kwargs):
            token, error = owner.token(scopes)
            if error:
                raise TokenError
            return func(owner, token, *args, **kwargs)

        return _wrapped_view

    return decorator



# Generated by Django 3.1.5 on 2021-01-25 10:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0018_auto_20210119_1701'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventimport',
            name='source',
            field=models.CharField(choices=[('Spectre Fleet', 'Spectre Fleet'), ('EVE University', 'EVE University'), ('Fun Inc.', 'Fun Inc.')], help_text='The API source where you want to pull events from', max_length=32),
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-04 12:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0005_auto_20201104_1127'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventhost',
            name='details',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='discord',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='facebook',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='fleet_comms',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='fleet_doctrines',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='ingame_channel',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='ingame_mailing_list',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='twitch',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='website',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='eventhost',
            name='youtube',
            field=models.CharField(blank=True, max_length=150),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-25 10:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0019_auto_20210125_1005'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='eventimport',
            options={'verbose_name': 'NPSI Event Import', 'verbose_name_plural': 'NPSI Event Imports'},
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-13 08:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('opcalendar', '0013_eventimport_creator'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventimport',
            name='creator',
            field=models.ForeignKey(default='1', help_text='User that has been used to create the fleet (most often the superuser who manages the plugin)', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='eventimport',
            name='host',
            field=models.ForeignKey(default=1, help_text='The AA host that will be used for the pulled events', on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventhost'),
        ),
        migrations.AlterField(
            model_name='eventimport',
            name='operation_type',
            field=models.ForeignKey(help_text='Operation type and ticker that will be assigned for the pulled fleets', on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventcategory'),
        ),
        migrations.AlterField(
            model_name='eventimport',
            name='source',
            field=models.CharField(choices=[('Spectre Fleet', 'Spectre Fleet'), ('EVE University', 'EVE University')], help_text='The API source where you want to pull events from', max_length=32),
        ),
    ]



import requests
import json
from typing import Tuple
from datetime import timedelta

from django.db import models
from django.urls import reverse
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from esi.models import Token

from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger

from .providers import esi
from .decorators import fetch_token_for_owner

logger = get_extension_logger(__name__)

class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            ("view_public", "Can see public events"),
            ("view_member", "Can see member events"),
            ("view_ingame", "Can see events from ingame calendar"),
            ("create_event", "Can create and edit events"),
            ("manage_event", "Can delete and manage signups"),
            ("add_ingame_calendar_owner", "Can add ingame calendar feeds for their corporation"),
        )     

class WebHook(models.Model):
    """Discord Webhook for pings"""
    name = models.CharField(max_length=150)
    webhook_url = models.CharField(max_length=500)
    enabled = models.BooleanField()

    def send_embed(self, embed):
        custom_headers = {'Content-Type': 'application/json'}
        data = '{"embeds": [%s]}' % json.dumps(embed)
        r = requests.post(self.webhook_url, headers=custom_headers,
                            data=data)
        r.raise_for_status()

    class Meta:
        verbose_name = 'Webhook'
        verbose_name_plural = 'Webhooks'

    def __str__(self):
        return '{}'.format(self.name)

class EventSignal(models.Model):
    """Fleet Timer Create/Delete pings"""

    webhook = models.ForeignKey(WebHook, on_delete=models.CASCADE)

    ignore_past_fleets = models.BooleanField(default=True)

    def __str__(self):
        return 'Send Fleets to "{}"'.format(self.webhook.name)

    class Meta:
        verbose_name = 'Fleet Signal'
        verbose_name_plural = 'Fleet Signals'    

class EventHost(models.Model):
    """Fleet Timer Create/Delete pings"""

    community = models.CharField(max_length=150, null=False)
    ingame_channel = models.CharField(max_length=150, blank=True)
    ingame_mailing_list = models.CharField(max_length=150, blank=True)
    fleet_comms = models.CharField(max_length=150, blank=True)
    fleet_doctrines = models.CharField(max_length=150, blank=True)
    website = models.CharField(max_length=150, blank=True)
    discord = models.CharField(max_length=150, blank=True)
    twitch = models.CharField(max_length=150, blank=True)
    twitter = models.CharField(max_length=150, blank=True)
    youtube = models.CharField(max_length=150, blank=True)
    facebook = models.CharField(max_length=150, blank=True)
    details = models.CharField(max_length=150, blank=True)
   
    def __str__(self):
        return str(self.community)

    class Meta:
        verbose_name = 'Host'
        verbose_name_plural = 'Hosts'           

class EventCategory(models.Model):
    # Colors for calendar
    COLOR_BLUE = "blue"
    COLOR_GREEN = "green"
    COLOR_RED = "red"
    COLOR_ORANGE = "orange"  
    COLOR_GREY = "grey"
    COLOR_YELLOW = "yellow"
    COLOR_PURPLE = "purple"

    COLOR_CHOICES = (
        (COLOR_GREEN, _("Green")),
        (COLOR_RED, _("Red")),
        (COLOR_ORANGE, _("Orange")),
        (COLOR_BLUE, _("Blue")),
        (COLOR_GREY, _("Grey")),
        (COLOR_YELLOW, _("Yellow")),
        (COLOR_PURPLE, _("Purple")),

    )
    name = models.CharField(max_length=150)
    ticker = models.CharField(max_length=10) 
    color = models.CharField(max_length=6, choices=COLOR_CHOICES, default='green')

    class Meta:
        verbose_name = 'Category'
        verbose_name_plural = 'Categories'

    def __str__(self):
        return str(self.name)

class EventImport(models.Model):
    """NPSI IMPORT OPTIONS"""
    SPECTRE_FLEET = "Spectre Fleet"
    EVE_UNIVERSITY = "EVE University"
    FUN_INC = "Fun Inc."
    
    IMPORT_SOURCES = (
        (SPECTRE_FLEET, _("Spectre Fleet")),
        (EVE_UNIVERSITY, _("EVE University")),
        (FUN_INC, _("Fun Inc.")),
        )

    source = models.CharField(
        max_length=32,
        choices=IMPORT_SOURCES,
        help_text="The API source where you want to pull events from"
    )
    
    host = models.ForeignKey(
        EventHost, on_delete=models.CASCADE,
        default=1,
        help_text="The AA host that will be used for the pulled events"
    )
    operation_type = models.ForeignKey(
        EventCategory,
        on_delete=models.CASCADE,
        help_text="Operation type and ticker that will be assigned for the pulled fleets"
    )
    creator = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        default='1',
        help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)"
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character"
    )
   
    def __str__(self):
        return str(self.source)

    class Meta:
        verbose_name = 'NPSI Event Import'
        verbose_name_plural = 'NPSI Event Imports'   

class Event(models.Model):
    # visibility
    VISIBILITY_PUBLIC = "public"
    VISIBILITY_MEMBER = "member"
    VISIBILITY_EXTERNAL = "import"

    VISIBILITY_CHOICES = [
        (VISIBILITY_PUBLIC, _("Public access")),
        (VISIBILITY_MEMBER, _("Members only access")),
    ]


    operation_type = models.ForeignKey(
        EventCategory, 
        on_delete=models.CASCADE
    )
    title = models.CharField(
        max_length=200, 
        unique=False
    )
    host = models.ForeignKey(
        EventHost, 
        on_delete=models.CASCADE, 
        default=1
    )
    doctrine = models.CharField(
        max_length=254, 
        default="", 
        blank=True
    )
    formup_system = models.CharField(
        max_length=254, 
        default="", 
        blank=True
    )
    description = models.TextField()
    start_time = models.DateTimeField()
    end_time = models.DateTimeField()
    fc = models.CharField(
        max_length=254, 
        default=""
    )
    visibility = models.CharField(
        max_length=7,
        choices=VISIBILITY_CHOICES,
        default=VISIBILITY_PUBLIC,
        db_index=True,
    )
    created_date = models.DateTimeField(
        default=timezone.now
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL
    )
    user = models.ForeignKey(
        User, 
        on_delete=models.CASCADE
    )

    class Meta:
        unique_together = ['title', 'start_time']

    def duration(self):
        return self.end_time - self.start_time

    def __str__(self):
        return self.title
    
    def get_absolute_url(self):
        return reverse('opcalendar:event-detail', args=(self.id,))

    @property
    def get_html_url(self):
        url = reverse('opcalendar:event-detail', args=(self.id,))
        return f'{url}'

    @property
    def get_html_title(self):
        return f'{self.start_time.strftime("%H:%M")} - {self.end_time.strftime("%H:%M")} <i>{self.host.community}</i> <br> <b>{self.operation_type.ticker} {self.title}</b>'
    @property
    def get_html_operation_color(self):
        return f'{self.operation_type.color}'

    @property
    def get_html_visibility_color(self):
        if self.visibility=="public":
            return "purple"
        if self.visibility=="import":
            return "grey"
        if self.visibility=="member":   
            return "blue"

    def user_can_edit(self, user: user) -> bool:
        """Checks if the given user can edit this timer. Returns True or False"""
        return user.has_perm("opcalendar.manage_event") or (
            self.user == user and user.has_perm("opcalendar.create_event")
        )

class Owner(models.Model):
    """A corporation that holds the calendars"""

    ERROR_NONE = 0
    ERROR_TOKEN_INVALID = 1
    ERROR_TOKEN_EXPIRED = 2
    ERROR_INSUFFICIENT_PERMISSIONS = 3
    ERROR_NO_CHARACTER = 4
    ERROR_ESI_UNAVAILABLE = 5
    ERROR_OPERATION_MODE_MISMATCH = 6
    ERROR_UNKNOWN = 99

    ERRORS_LIST = [
        (ERROR_NONE, "No error"),
        (ERROR_TOKEN_INVALID, "Invalid token"),
        (ERROR_TOKEN_EXPIRED, "Expired token"),
        (ERROR_INSUFFICIENT_PERMISSIONS, "Insufficient permissions"),
        (ERROR_NO_CHARACTER, "No character set for fetching data from ESI"),
        (ERROR_ESI_UNAVAILABLE, "ESI API is currently unavailable"),
        (
            ERROR_OPERATION_MODE_MISMATCH,
            "Operaton mode does not match with current setting",
        ),
        (ERROR_UNKNOWN, "Unknown error"),
    ]
    
    corporation = models.OneToOneField(
        EveCorporationInfo,
        default=None,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        help_text="Corporation owning the calendar",
        related_name="+",
    )
    character = models.ForeignKey(
        CharacterOwnership,
        on_delete=models.SET_DEFAULT,
        default=None,
        null=True,
        blank=True,
        help_text="Character used for syncing the calendar",
        related_name="+",
    )
    is_active = models.BooleanField(
        default=True,
        help_text=("whether this owner is currently included in the sync process"),
    )

    class Meta:
        verbose_name = 'Ingame Clanedar Owner'
        verbose_name_plural = 'Ingame Calendar Owners'  

    def update_events_esi(self):
        if self.is_active:
            
            #Get all current imported fleets in database
            event_ids_to_remove = list(
                IngameEvents.objects.filter(owner=self).values_list("event_id", flat=True)
            )
            logger.debug("Ingame events currently in database: %s" % event_ids_to_remove)
            
            events = self._fetch_events()
            token = self.token(
                [
                    "esi-calendar.read_calendar_events.v1",
                ]
            )[0]
            
            for event in events:   

                character_id = self.character.character.character_id
               
                details = esi.client.Calendar.get_characters_character_id_calendar_event_id(
                    character_id=character_id,
                    event_id=event["event_id"],
                    token=token.valid_access_token(),
                ).results()

                end_date = event["event_date"] + timedelta(minutes=details['duration'])

                original = IngameEvents.objects.filter(
                    owner=self, event_id=event["event_id"]
                ).first()

                if original is not None:
                    
                    logger.debug("Event: %s already in database" % event["title"])
                    event_ids_to_remove.remove(original.event_id)

                else:
                    IngameEvents.objects.create(
                        event_id=event["event_id"],
                        owner = self,
                        text = details['text'],
                        owner_type = details['owner_type'],
                        owner_name = details['owner_name'],
                        importance = details['importance'],
                        duration = details['duration'],
                        event_start_date=event["event_date"],
                        event_end_date=end_date,
                        title=event["title"],
        
                    )
                    logger.debug("New event created: %s" % event["title"])

            logger.debug("Removing all events that we did not get over API")
            IngameEvents.objects.filter(pk__in=event_ids_to_remove).delete()

            logger.debug("All events fetched for %s" % self.character.character.character_name)

    @fetch_token_for_owner(["esi-calendar.read_calendar_events.v1"])
    def _fetch_events(self, token) -> list:

        character_id = self.character.character.character_id

        events = esi.client.Calendar.get_characters_character_id_calendar(
            character_id=character_id,
            token=token.valid_access_token(),
        ).results()
        
        return events

    def token(self, scopes=None) -> Tuple[Token, int]:
        """returns a valid Token for the owner"""
        token = None
        error = None
        add_prefix = ""

        # abort if character is not configured
        if self.character is None:
            logger.error(add_prefix("No character configured to sync"))
            error = self.ERROR_NO_CHARACTER

        # abort if character does not have sufficient permissions
        elif self.corporation and not self.character.user.has_perm(
            "opcalendar.add_ingame_calendar_owner"
        ):
            logger.error(
                add_prefix(
                    "This character does not have sufficient permission to sync corporation calendars"
                )
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        # abort if character does not have sufficient permissions
        elif not self.character.user.has_perm(
            "opcalendar.add_ingame_calendar_owner"
        ):
            logger.error(
                add_prefix(
                    "This character does not have sufficient permission to sync personal calendars"
                )
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        else:
            try:
                # get token
                token = (
                    Token.objects.filter(
                        user=self.character.user,
                        character_id=self.character.character.character_id,
                    )
                    .require_scopes(scopes)
                    .require_valid()
                    .first()
                )
            except TokenInvalidError:
                logger.error(add_prefix("Invalid token for fetching calendars"))
                error = self.ERROR_TOKEN_INVALID
            except TokenExpiredError:
                logger.error(add_prefix("Token expired for fetching calendars"))
                error = self.ERROR_TOKEN_EXPIRED
            else:
                if not token:
                    logger.error(add_prefix("No token found with sufficient scopes"))
                    error = self.ERROR_TOKEN_INVALID

        return token, error


class IngameEvents(models.Model):

    event_id = models.PositiveBigIntegerField(
        primary_key=True, help_text="The EVE ID of the event"
    )
    owner = models.ForeignKey(
        Owner,
        on_delete=models.CASCADE,
        help_text="Event holder",
    )
    event_start_date = models.DateTimeField()
    event_end_date = models.DateTimeField(
        blank=True,
        null=True
    )
    title = models.CharField(
        max_length=128
    )
    text = models.TextField()
    owner_type = models.CharField(
        max_length=128
    )
    owner_name = models.CharField(
        max_length=128
    )
    importance = models.CharField(
        max_length=128
    )
    duration = models.CharField(
        max_length=128
    )

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = 'Ingame Event'
        verbose_name_plural = 'Ingame Events' 

    def get_absolute_url(self):
        return reverse('opcalendar:ingame-event-detail', args=(self.event_id,))

    @property
    def get_html_url(self):
        url = reverse('opcalendar:ingame-event-detail', args=(self.event_id,))
        return f'{url}'

    @property
    def get_html_operation_color(self):
        return "black"
    
    @property
    def get_html_title(self):
        return f'{self.event_start_date.strftime("%H:%M")} - {self.event_end_date.strftime("%H:%M")}<i> {self.owner_name}</i><br><b>{self.title}</b>'
    

class EventMember(models.Model):
    event = models.ForeignKey(Event, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    def __str__(self):
        return "%s for %s" % (self.__class__.__name__, self.corp)

    class Meta:
        unique_together = ['event', 'user']

    def __str__(self):
        return str(self.user)

    



# Generated by Django 3.1.2 on 2021-01-19 15:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0012_index_additions'),
        ('authentication', '0017_remove_fleetup_permission'),
        ('opcalendar', '0015_eventimport_eve_character'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('basic_access', 'Can access this app'), ('view_public', 'Can see public events'), ('view_member', 'Can see member events'), ('view_ingame', 'Can see events from ingame calendar'), ('create_event', 'Can create and edit events'), ('manage_event', 'Can delete and manage signups'), ('add_ingame_calendar_owner', 'Can add ingame calendar feeds for their corporation'))},
        ),
        migrations.CreateModel(
            name='Owner',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, help_text='whether this owner is currently included in the sync process')),
                ('character', models.ForeignKey(blank=True, default=None, help_text='character used for syncing blueprints', null=True, on_delete=django.db.models.deletion.SET_DEFAULT, related_name='+', to='authentication.characterownership')),
                ('corporation', models.OneToOneField(blank=True, default=None, help_text="Corporation owning blueprints, if this is a 'corporate' owner", null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecorporationinfo')),
            ],
        ),
        migrations.CreateModel(
            name='IngameEvents',
            fields=[
                ('event_id', models.PositiveBigIntegerField(help_text='The EVE ID of the event', primary_key=True, serialize=False)),
                ('event_start_date', models.DateTimeField()),
                ('event_end_date', models.DateTimeField(blank=True, null=True)),
                ('title', models.CharField(max_length=128)),
                ('text', models.TextField()),
                ('owner_type', models.CharField(max_length=128)),
                ('owner_name', models.CharField(max_length=128)),
                ('importance', models.CharField(max_length=128)),
                ('duration', models.CharField(max_length=128)),
                ('owner', models.ForeignKey(help_text='Event holder', on_delete=django.db.models.deletion.CASCADE, to='opcalendar.owner')),
            ],
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-04 10:42

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0012_index_additions'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access this app'), ('view_public', 'Can see public events'), ('view_member', 'Can see member events'), ('create_event', 'Can create and edit events'), ('manage_event', 'Can delete and manage signups')),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='EventCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('ticker', models.CharField(max_length=10)),
                ('color', models.CharField(choices=[('green', 'Green'), ('red', 'Red'), ('orange', 'Orange'), ('blue', 'Blue'), ('grey', 'Grey'), ('yellow', 'Yellow')], default='green', max_length=6)),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
        ),
        migrations.CreateModel(
            name='WebHook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('webhook_url', models.CharField(max_length=500)),
                ('enabled', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Webhook',
                'verbose_name_plural': 'Webhooks',
            },
        ),
        migrations.CreateModel(
            name='EventSignal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ignore_past_fleets', models.BooleanField(default=True)),
                ('webhook', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='opcalendar.webhook')),
            ],
            options={
                'verbose_name': 'Fleet Signal',
                'verbose_name_plural': 'Fleet Signals',
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('doctrine', models.CharField(default='', max_length=254)),
                ('formup_system', models.CharField(default='', max_length=254)),
                ('description', models.TextField()),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField()),
                ('fc', models.CharField(default='', max_length=254)),
                ('visibility', models.CharField(choices=[('Public', 'Public access'), ('Member', 'Members only access')], db_index=True, default='Public', max_length=7)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('eve_character', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.evecharacter')),
                ('operation_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventcategory')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EventMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='opcalendar.event')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('event', 'user')},
            },
        ),
    ]



from django.contrib import admin
from opcalendar.models import Event, EventCategory, EventSignal, WebHook, EventHost, EventImport, Owner, IngameEvents

from django.utils.translation import ugettext_lazy as _

class WebHookAdmin(admin.ModelAdmin):
    list_display=('name', 'enabled')

admin.site.register(WebHook, WebHookAdmin)

class EventSignalAdmin(admin.ModelAdmin):
	list_display=('get_webhook','ignore_past_fleets')

	def get_webhook(self, obj):
		return obj.webhook.name
	get_webhook.short_description = 'Webhook Name'
	get_webhook.admin_order_field = 'webhook__name'

admin.site.register(EventSignal, EventSignalAdmin)

class EventCategoryAdmin(admin.ModelAdmin):
    model = EventCategory

admin.site.register(EventCategory, EventCategoryAdmin)

class EventHostAdmin(admin.ModelAdmin):
    model = EventHost

admin.site.register(EventHost, EventHostAdmin)

class EventHostAdmin(admin.ModelAdmin):
    model = EventImport

admin.site.register(EventImport, EventHostAdmin)


admin.site.register(Owner)

admin.site.register(IngameEvents)

admin.site.register(Event)





# Generated by Django 3.1.2 on 2021-01-19 17:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0017_auto_20210119_1626'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ingameevents',
            options={'verbose_name': 'Ingame Event', 'verbose_name_plural': 'Ingame Events'},
        ),
        migrations.AlterModelOptions(
            name='owner',
            options={'verbose_name': 'Ingame Clanedar Owner', 'verbose_name_plural': 'Ingame Calendar Owners'},
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-04 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventcategory',
            name='color',
            field=models.CharField(choices=[('green', 'Green'), ('red', 'Red'), ('orange', 'Orange'), ('blue', 'Blue'), ('grey', 'Grey'), ('yellow', 'Yellow'), ('purple', 'Purple')], default='green', max_length=6),
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-08 08:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0010_auto_20201105_1602'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventImport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('source', models.CharField(choices=[('Spectre Fleet', 'Spectre Fleet')], max_length=32)),
                ('host', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventhost')),
                ('operation_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventcategory')),
            ],
            options={
                'verbose_name': 'Event Import',
                'verbose_name_plural': 'Event Imports',
            },
        ),
    ]



# cal/views.py
import logging
import calendar
from datetime import datetime, date, timedelta

from django.contrib import messages

from django.views import generic
from django.views.generic import ListView, UpdateView, DeleteView
from django.shortcuts import render, redirect
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.urls import reverse_lazy
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy
from django.utils.translation import ugettext_lazy as _
from django.db import transaction

from esi.decorators import token_required

from allianceauth.services.hooks import get_extension_logger

from . import tasks
from .models import *
from .utils import Calendar, messages_plus
from .forms import EventForm, AddMemberForm

logger = get_extension_logger(__name__)

@login_required(login_url='signup')
def index(request):
    return HttpResponse('hello')

@token_required(scopes=['esi-calendar.read_calendar_events.v1'])
def add_ingame_calendar(request, token):
    token_char = EveCharacter.objects.get(character_id=token.character_id)

    success = True
    try:
        owned_char = CharacterOwnership.objects.get(
            user=request.user, character=token_char
        )
    except CharacterOwnership.DoesNotExist:

        success = False
        owned_char = None

        messages_plus.error(
            request,
            format_html(
                gettext_lazy(
                    "You can only use your main or alt characters "
                    "to add corporations. "
                    "However, character %s is neither. "
                )
                % format_html("<strong>{}</strong>", token_char.character_name)
            ),
        )


    if success:

        try:
            corporation = EveCorporationInfo.objects.get(
                corporation_id=token_char.corporation_id
            )
        except EveCorporationInfo.DoesNotExist:
            corporation = EveCorporationInfo.objects.create_corporation(
                token_char.corporation_id
            )

        with transaction.atomic():
            owner, _ = Owner.objects.update_or_create(
                corporation=corporation, defaults={"character": owned_char}
            )

            owner.save()

        tasks.update_events_for_owner.delay(owner_pk=owner.pk)
        
        messages_plus.success(
            request,
            format_html(
                gettext_lazy(
                    "Succesfully added ingame calendar sync for %(character)s. Process will run on background. You will receive a report once the process is finished."
                )
                % {
                    "corporation": format_html("<strong>{}</strong>", owner),
                    "character": format_html(
                        "<strong>{}</strong>", owner.character.character.character_name
                    ),
                }
            ),
        )

    return redirect("opcalendar:calendar")

def get_date(req_day):
    if req_day:
        year, month = (int(x) for x in req_day.split('-'))
        return date(year, month, day=1)
    return datetime.today()

def prev_month(d):
    first = d.replace(day=1)
    prev_month = first - timedelta(days=1)
    month = 'month=' + str(prev_month.year) + '-' + str(prev_month.month)
    return month

def next_month(d):
    days_in_month = calendar.monthrange(d.year, d.month)[1]
    last = d.replace(day=days_in_month)
    next_month = last + timedelta(days=1)
    month = 'month=' + str(next_month.year) + '-' + str(next_month.month)
    return month

class CalendarView(LoginRequiredMixin, PermissionRequiredMixin, ListView):
    permission_required = 'opcalendar.basic_access'
    login_url = 'signup'
    model = Event
    template_name = 'opcalendar/calendar.html'

    def get_context_data(self, **kwargs):
        user = self.request.user
        #user_permissions = self.request.user.has_perm('app.get_main_view')
        context = super().get_context_data(**kwargs)
        d = get_date(self.request.GET.get('month', None))
        cal = Calendar(d.year, d.month, user)
        html_cal = cal.formatmonth(withyear=True)
        context['category'] = EventCategory.objects.all()
        context['calendar'] = mark_safe(html_cal)
        context['prev_month'] = prev_month(d)
        context['next_month'] = next_month(d)
        context['legend'] = "test"
        return context

@login_required(login_url='signup')
@permission_required("opcalendar.create_event")
def create_event(request):    
    form = EventForm(request.POST or None)
    if request.POST and form.is_valid():
        # Get character
        character = request.user.profile.main_character
        operation_type = form.cleaned_data['operation_type']
        title = form.cleaned_data['title']
        doctrine = form.cleaned_data['doctrine']
        formup_system = form.cleaned_data['formup_system']
        description = form.cleaned_data['description']
        start_time = form.cleaned_data['start_time']
        end_time = form.cleaned_data['end_time']
        fc = form.cleaned_data['fc']
        visibility = form.cleaned_data['visibility']
        Event.objects.get_or_create(
            user=request.user,
            operation_type=operation_type,
            title=title,
            doctrine=doctrine,
            formup_system=formup_system,
            description=description,
            start_time=start_time,
            end_time=end_time,
            eve_character = character,
            fc=fc,
            visibility=visibility
        )
        messages.success(request, _('Event %(opname)s created for %(date)s.') % {"opname": title,"date": start_time.strftime("%Y-%m-%d %H:%M")})
        
        return HttpResponseRedirect(reverse('opcalendar:calendar'))
   
    return render(request, 'opcalendar/event-add.html', {'form': form})



@login_required(login_url='signup')
@permission_required("opcalendar.basic_access")
def event_details(request, event_id):
    
    event = Event.objects.get(id=event_id)
    
    context = {
        'event': event
    }

    if request.user.has_perm('opcalendar.view_public') and event.visibility == "public" or event.visibility == "import":
        return render(request, 'opcalendar/event-details.html', context)
    elif request.user.has_perm('opcalendar.view_member') and event.visibility == "member":
        return render(request, 'opcalendar/event-details.html', context) 
    else:
        return redirect('opcalendar:calendar')

class EventEdit(generic.UpdateView):
    permission_required = "opcalendar.create_event"
    model = Event
    fields = ['operation_type', 'title', 'doctrine', 'formup_system', 'description', 'start_time','end_time','fc', 'visibility']
    template_name = 'opcalendar/event-edit.html'

@login_required(login_url='signup')
@permission_required("opcalendar.basic_access")
def ingame_event_details(request, event_id):
    event = IngameEvents.objects.get(event_id=event_id)
    
    context = {
        'event': event
    }

    if request.user.has_perm('opcalendar.view_ingame'):
        return render(request, 'opcalendar/ingame-event-details.html', context)
    else:
        return redirect('opcalendar:calendar') 

@login_required(login_url='signup')
@permission_required("opcalendar.manage_event")
def add_eventmember(request, event_id):
    forms = AddMemberForm()
    if request.method == 'POST':
        forms = AddMemberForm(request.POST)
        if forms.is_valid():
            member = EventMember.objects.filter(event=event_id)
            event = Event.objects.get(id=event_id)
            if member.count() <= 9:
                user = forms.cleaned_data['user']
                EventMember.objects.create(
                    event=event,
                    user=user
                )
                return redirect('opcalendar:calendar')
            else:
                print('--------------User limit exceed!-----------------')
    context = {
        'form': forms
    }
    return render(request, 'opcalendar/add_member.html', context)

@login_required
@permission_required('opcalendar.create_event')
def EventDeleteView(request, event_id):
    logger.debug("remove_optimer called by user %s for operation id %s" % (request.user, event_id))
    op = get_object_or_404(Event, id=event_id)
    op.delete()
    logger.info("Deleting optimer id %s by user %s" % (event_id, request.user))
    messages.error(request, _('Removed event %(opname)s.') % {"opname": op.title})
    return redirect("opcalendar:calendar")


# Generated by Django 3.1.2 on 2020-11-05 08:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0007_eventhost_twitter'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='event',
            unique_together={('title', 'start_time')},
        ),
    ]



from django.shortcuts import render, redirect
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.models import User
from opcalendar.forms import SignupForm

def signup(request):
    forms = SignupForm()
    if request.method == 'POST':
        forms = SignupForm(request.POST)
        if forms.is_valid():
            username = forms.cleaned_data['username']
            password = forms.cleaned_data['password']
            user = authenticate(username=username, password=password)
            if user:
                login(request, user)
                return redirect('opcalendar:calendar')
    context = {'form': forms}
    return render(request, 'signup.html', context)


def user_logout(request):
    logout(request)
    return redirect('signup')


# Generated by Django 3.1.2 on 2020-11-05 16:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0009_auto_20201105_0859'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='visibility',
            field=models.CharField(choices=[('public', 'Public access'), ('member', 'Members only access')], db_index=True, default='public', max_length=7),
        ),
    ]
